var isInt = require('is-integer');
var Node, HalfEdgE;

/**
 * Edge object
 * @constructor
 * @param {Node} node1 - start node
 */
var Edge = module.exports = function(node1, node2, weight1, weight2){
	Node = require('./node.js');
	HalfEdge = require('./halfedge.js');
	if(!(this instanceof Edge)){
		return new Edge(node1, node2, weight1, weight2);
	}

	if(!(node1 instanceof Node && node2 instanceof Node)){
		throw new Error("first two arguments to Edge contructor must be Node objects");
	}

	if(weight1 === undefined || weight1 === false && weight2 === undefined){
		//normal edge
		weight1 = 1;
		weight2 = 1;
	}else if(weight1 === true){
		//directional edge
		weight1 = 1;
		weight2 = null;
	}else if(isInt(weight1) && weight2 === true){
		//weighted directional edge
		weight1 = weight1;
		weight2 = null;
	}else if(isInt(weight1) && weight2 === undefined || weight2 === false){
		//weighted non-directional edge
		weight1 = weight1;
		weight2 = weight1;
	}else if(isInt(weight1) && isInt(weight2)){
		//weighted bi-directional edge
	}else{
		throw new Error("Incorrect arguments given to new Edge()");
	}

	(function(){
		this.nodes = [node1, node2];
		this.weights = [weight1, weight2];

		node1.edges.push(this);
		node2.edges.push(this);
	}).call(this);
};

Edge.prototype = {
	/**
	 * Get a node which is connected to another node by this edge
	 * @memberof Edge#
	 * @param {Node} node which is connected to this edge
	 * @returns {Node} node which is connected to the node parameter
	 * @throws Will throw an error if the node parameter isn't connected to this edge
	 */
	getNeighbourByNode : function(node){
		if(!(node instanceof Node)){
			throw new Error("first argument to getNeighbourByNode() must be a Node object");
		}

		if(this.nodes[0] === node){
			return this.nodes[1];
		}else if(this.nodes[1] === node){
			return this.nodes[0];
		}else{
			throw new Error("first argument Node object is not connected to the Edge");
		}
	},

	/**
	 * Get the weight of this edge
	 * @memberof Edge#
	 * @param {Node} node - node that is connected to this edge
	 * @returns {Int} weight of this edge in the direction of the node parameter
	 * @throws Will throw an error if the node parameter isn't connected to this edge
	 */
	getWeightToNode : function(node){
		if(!(node instanceof Node)){
			throw new Error("first argument to getWeightToNode() must be a Node object");
		}

		if(this.nodes[0] === node){
			return this.weights[1];
		}else if(this.nodes[1] === node){
			return this.weights[0];
		}else{
			throw new Error("first argument Node object is not connected to the Edge");
		}
	},

	/**
	 * Get a HalfEdge object which is directed towards a given node
	 * @memberof Edge#
	 * @param {Node} node - node that is connected to this edge
	 * @returns {HalfEdge} HalfEdge object that represents the half of this edge which is
	 * directed towards the node parameter
	 */
	getHalfEdge : function(node){
		var halfEdge = new HalfEdge(this, node);
		if(halfEdge.weight === null){
			return null;
		}else{
			return halfEdge;
		}
	},
};
