var assert = require('assert');
var bacon = require('../src/index.js');

describe('autocreate functions', function(){
	describe('bacon.autocreate()', function(){
		it('expects an array or object', function(){
			assert.doesNotThrow(function(){
				bacon.autocreate([]);
			});
			assert.doesNotThrow(function(){
				bacon.autocreate({});
			});
			assert.throws(function(){
				bacon.autocreate(null);
			});
			assert.throws(function(){
				bacon.autocreate();
			});
			assert.throws(function(){
				bacon.autocreate(2);
			});
		});
		it('returns a new graph object', function(){
			var g = bacon.autocreate([[],[]]);
			assert(g instanceof bacon.Graph);
		});
	});
	describe('create single graphs', function(){
		describe('simple graph with 4 unconnected nodes', function(){
			var data = [
				[],
				[],
				[],
				[]
			];
			var graph = bacon.autocreate(data);
			it('has 4 nodes', function(){
				assert.strictEqual(4, graph.nNodes());
			});
			it('has correct number of edges', function(){
				assert.strictEqual(0, graph.getNode(0).nEdges());
				assert.strictEqual(0, graph.getNode(1).nEdges());
				assert.strictEqual(0, graph.getNode(2).nEdges());
				assert.strictEqual(0, graph.getNode(3).nEdges());
			});
		});
		describe('simple graph with 4 connected nodes', function(){
			var data = [
				[1],
				[],
				[3,0],
				[0]
			];
			var graph = bacon.autocreate(data);
			it('has 4 nodes', function(){
				assert.strictEqual(4, graph.nNodes());
			});
			it('has correct number of edges', function(){
				assert.strictEqual(3, graph.getNode(0).nEdges());
				assert.strictEqual(1, graph.getNode(1).nEdges());
				assert.strictEqual(2, graph.getNode(2).nEdges());
				assert.strictEqual(2, graph.getNode(3).nEdges());
			});
		});
		describe('graph with repeat edges', function(){
			var graph = bacon.autocreate([
				[1,2,3],
				[0,0,3],
				[0],
				[0]
			]);
			it('has 4 nodes', function(){
				assert.strictEqual(4, graph.nNodes());
			});
			it('has correct number of edges', function(){
				assert.strictEqual(7, graph.getNode(0).nEdges());
				assert.strictEqual(4, graph.getNode(1).nEdges());
				assert.strictEqual(2, graph.getNode(2).nEdges());
				assert.strictEqual(3, graph.getNode(3).nEdges());
			});
		});
	});
	describe('creates multiple graphs', function(){
		describe('returns an object', function(){
			var graphs = bacon.autocreate({
				'graph1'  : [[],[]],
				'graph2'  : [[1],[],[0]],
				'u_score' : [],
				'g-raph'  : []
			});
			it('containing graphs', function(){
				assert(graphs.graph1 instanceof bacon.Graph);
				assert(graphs.graph2 instanceof bacon.Graph);
				assert(graphs.u_score instanceof bacon.Graph);
				assert(graphs['g-raph'] instanceof bacon.Graph);
			});
			it('of graphs which have the correct properties', function(){
				assert.strictEqual(2, graphs.graph1.nNodes());
				assert.strictEqual(0, graphs.graph1.getNode(0).nEdges());
				assert.strictEqual(3, graphs.graph2.nNodes());
				assert.strictEqual(2, graphs.graph2.getNode(0).nEdges());
			});
		});
	});

	describe('directed graph option', function(){
		var data = [
				[1,3],
				[],
				[3,0],
				[0]
		];
		it('directed = true', function(){
			var graph = bacon.autocreate(data, {
				directed: true,
			});
			assert(graph instanceof bacon.Graph);
			assert(graph.getNode(0).isNeighbour(graph.getNode(1)), "0 -> 1");
			assert(!graph.getNode(1).isNeighbour(graph.getNode(0)), "1 !-> 0")
			assert(graph.getNode(2).isNeighbour(graph.getNode(0)), "2 -> 0")
			assert(!graph.getNode(0).isNeighbour(graph.getNode(2)), "0 !-> 2")
			assert(!graph.getNode(1).isNeighbour(graph.getNode(2)), "1 !-> 2")

			assert(graph.getNode(3).isNeighbour(graph.getNode(0)), "3 -> 0");
			assert(graph.getNode(0).isNeighbour(graph.getNode(3)), "0 -> 3");

			assert.strictEqual(4, graph.getNode(0).nEdges());
			assert.strictEqual(1, graph.getNode(1).nEdges());
			assert.strictEqual(2, graph.getNode(2).nEdges());
			assert.strictEqual(3, graph.getNode(3).nEdges());
		});
		it('directed = false', function(){
			var graph = bacon.autocreate(data, {
				directed: false,
			});
			assert(graph instanceof bacon.Graph);
			assert(graph.getNode(0).isNeighbour(graph.getNode(1)), "0 -> 1");
			assert(graph.getNode(1).isNeighbour(graph.getNode(0)), "1 -> 0")
			assert(graph.getNode(2).isNeighbour(graph.getNode(0)), "2 -> 0")
			assert(graph.getNode(0).isNeighbour(graph.getNode(2)), "0 -> 2")
			assert(!graph.getNode(1).isNeighbour(graph.getNode(2)), "1 !-> 2")

			assert(graph.getNode(3).isNeighbour(graph.getNode(0)), "3 -> 0");
			assert(graph.getNode(0).isNeighbour(graph.getNode(3)), "0 -> 3");

			assert.strictEqual(4, graph.getNode(0).nEdges());
			assert.strictEqual(1, graph.getNode(1).nEdges());
			assert.strictEqual(2, graph.getNode(2).nEdges());
			assert.strictEqual(3, graph.getNode(3).nEdges());
		});
	});
});
