var assert = require('assert');
var bacon = require('../src/index.js');

describe('Edge', function(){
	describe('creation', function(){
		it('constructor expects two nodes as it\'s arguments', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			assert.doesNotThrow(function(){
				var edge = new bacon.Edge(node, node2);
			});
			assert.throws(function(){
				var edge = new bacon.Edge(node, {});
			}, /first two arguments to Edge contructor must be Node objects/);
			assert.throws(function(){
				var edge = new bacon.Edge({}, node2);
			}, /first two arguments to Edge contructor must be Node objects/);
			assert.throws(function(){
				var edge = new bacon.Edge();
			}, /first two arguments to Edge contructor must be Node objects/);
		});
		it('returns a edge object', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			var edge = new bacon.Edge(node, node2);
			assert(edge instanceof bacon.Edge);
		});
		it('constructor doesn\'t need the \'new\' keyword', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			var edge = bacon.Edge(node, node2);
			assert(edge instanceof bacon.Edge);
		});
		it('allows new edge, even if nodes are already connected', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			var edge = bacon.Edge(node, node2);
			assert.doesNotThrow(function(){
				var repeat_edge = bacon.Edge(node, node2);
			});
		});
	});
	describe('getNeighbourByNode', function(){
		it('expects a node as it\'s first argument', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			var edge = bacon.Edge(node, node2);
			assert.throws(function(){
				edge.getNeighbourByNode();
			}, /first argument to getNeighbourByNode\(\) must be a Node object/);
			assert.throws(function(){
				edge.getNeighbourByNode({});
			}, /first argument to getNeighbourByNode\(\) must be a Node object/);
			assert.doesNotThrow(function(){
				edge.getNeighbourByNode(node);
				edge.getNeighbourByNode(node2);
			});
		});
		it('throws an error if the node passed as first argument isn\'t conected to the edge', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			var node3 = bacon.Node();
			var edge = bacon.Edge(node, node2);
			assert.throws(function(){
				edge.getNeighbourByNode(node3);
			}, /first argument Node object is not connected to the Edge/);
		});
		it('returns a node\'s neighbour', function(){
			var node = bacon.Node();
			var node2 = bacon.Node();
			var edge = bacon.Edge(node, node2);
			assert.deepEqual(node2, edge.getNeighbourByNode(node));
		});
	});
});
